/*
    This file is part of FileDown
 
    FileDown is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <spectrum.h>
#include <input.h>
#include <stdlib.h>
#include <time.h>

#include "utilsTexto.h"

#define ZXUNO_ADDR 64571
#define ZXUNO_REG 64827
#define UART_DATA_REG 250
#define UART_STAT_REG 251
#define BYTE_RECEIVED_BIT 0x80
#define BYTE_TRANSMTTING_BIT 0x40

#define getTecla() in_Inkey()
void esperarTecla();

void iniciarUART();
void enviarByteUART( unsigned char valor );
unsigned char recibirByteUART();
//unsigned char recibirByteSinBloqueoUART();
unsigned char recibirByteSinBloqueoUART( unsigned char *byteRecibido );



//*****************************************
void main(void) {

    unsigned char caracter, tecla;

    iniciarUART();

    modoZX32ColumnasTexto();
    clsTexto();

    imprimir( "Empezando.\n");

    while ( 1 ) {

        if ( recibirByteSinBloqueoUART( &caracter ) != 0 ) {
            // Byte recibido, imprimirlo en pantalla
            fputc_cons( caracter );
        }

        tecla = getTecla();
        if ( tecla ) {
            
            while ( getTecla() ) {
                // Esperar a que se suelte la tecla
            }

            if ( tecla == 13 ) {
                // El chip WiFi necesita 13, 10 para aceptar un comando (fin de linea)
                enviarByteUART( 13 );
                enviarByteUART( 10 );

            }
            else {
                enviarByteUART( tecla );
            }

        }

    }

}
//*****************************************



void iniciarUART() {

    // Limpia los flags de la uart leyendo los dos registros

    outp( ZXUNO_ADDR, UART_STAT_REG );
    inp( ZXUNO_REG );

    outp( ZXUNO_ADDR, UART_DATA_REG );
    inp( ZXUNO_REG );

}

void enviarByteUART( unsigned char valor ) {

    unsigned char t;

    outp( ZXUNO_ADDR, UART_STAT_REG );
    while ( ( inp( ZXUNO_REG ) & BYTE_TRANSMTTING_BIT ) == BYTE_TRANSMTTING_BIT ) {
        // Espera a que se termine de transmitir el byte anterior
    }

    outp( ZXUNO_ADDR, UART_DATA_REG );
    outp( ZXUNO_REG, valor );

}

unsigned char recibirByteUART() {

    // Esta funcion es bloqueante, no termina hasta que se recibe un byte por el puerto serie.

    unsigned char valor;
    unsigned char t;

    outp( ZXUNO_ADDR, UART_STAT_REG );
    while ( ( inp( ZXUNO_REG ) & BYTE_RECEIVED_BIT ) == 0 ) {
        // Espera a recibir un byte, no hace nada
    }

    outp( ZXUNO_ADDR, UART_DATA_REG );
    valor = inp( ZXUNO_REG );

    return valor;

}

unsigned char recibirByteSinBloqueoUART( unsigned char *byteRecibido ) {

    // Esta funcion mira sin bloquearse si se ha recibido un byte por el puerto serie.
    // En caso afirmativo devuelve 1 y en byteRecibido se obtiene el valor.
    // En caso contrario devuelve 0.

    outp( ZXUNO_ADDR, UART_STAT_REG );
    if ( ( inp( ZXUNO_REG ) & BYTE_RECEIVED_BIT ) == BYTE_RECEIVED_BIT ) {

        outp( ZXUNO_ADDR, UART_DATA_REG );
        *byteRecibido = inp( ZXUNO_REG );
        return 1;

    }

    return 0;

}

void esperarTecla() {
    while ( getTecla() );
    while ( ! getTecla() );
}
